//Block.java - block
//Copyright (c) 2007 Thomas Schneider
//See full Copyright license terms in file ../GPL.txt

package SFE.PF;

import SFE.BOAL.Gate;

/**
 * A Block is a programmable sub-circuit.
 *
 * @author Thomas Schneider
 */
public abstract class Block {
	/** #inputs */
	protected int u;

	/** #outputs */
	protected int v;

	boolean debug;

	/**
	 * Extract circuit representation of block.
	 * 
	 * @param inputs input wires
	 * @param outputs buffer for output wires
	 * @return output wires (outputs or newly allocated buffer if outputs=null)
	 */
	abstract public Gate[] extractCircuit(Gate[] inputs, Gate[] outputs);

	/**
	 * Program block.
	 * 
	 * @param p program - semantics depend on type of block
	 */
	abstract public void program(int p[]);
	
	/**
	 * Count number of gates of this block.
	 * 
	 * @return #gates
	 */
	abstract public int countGates();

	/**
	 * Constructor
	 * 
	 * @param u #inputs
	 * @param v #outputs
	 * @param c UC containing this selection block
	 */
	public Block(int u, int v, UC c) {
		this.u=u;
		this.v=v;
		this.debug=(c==null);
	}
	
	/**
	 * Dummy Constructor
	 */
	public Block() {}
}
