//S_u_1.java - S(u,1) selection block
//Copyright (c) 2007 Thomas Schneider
//See full Copyright license terms in file ../GPL.txt

package SFE.PF;

import SFE.BOAL.Gate;
import SFE.PF.UC.Y;

/**
 * S(u,1) selection block.
 *
 * @author Thomas Schneider
 */
public class S_u_1 extends S {
	Y s[];

	public int countGates() {
		return (u-1)*UC.Ygates;
	}
	
	public int estimateGates(int u, int v) {
		if(v!=1) return Integer.MAX_VALUE;
		else return (u-1)*UC.Ygates;
	}

	public void program(int[] select) {
		assert(select.length==1);
		assert(select[0]>=0 && select[0]<u);
		
		if(u==1) return;

		if(s[0]!=null) {
			for(int i=0; i<u-1; i++) {
				if(i+1==select[0]) s[i].program(true);
				else s[i].program(false);
			}
		}
		else System.out.println("Selecting input "+select[0]);
	}
	
	public void program(int select) {
		program(new int[]{select});
	}
 
	public Gate[] extractCircuit(Gate[] inputs, Gate[] outputs) {
		if(outputs==null) outputs=new Gate[1];
		if(u==1) {
			outputs[0]=inputs[0];
			return outputs;
		}

		Gate last=s[0].extractCircuit(inputs[0], inputs[1]);
		for(int i=1; i<u-1; i++) {
			last=s[i].extractCircuit(last, inputs[i+1]);
		}
		outputs[0]=last;
		
		return outputs;
	}
	
	public S_u_1(int u, UC c) {
		super(u,1,c);
	}
	public S_u_1() {
		super();
	}
	
	public void generate(int u, int v, UC c) {
		assert(v==1);
		this.u=u; this.v=v;
		if(u==1) return;
		s=new Y[u-1];
		if(c!=null) {
			for(int i=0; i<u-1; i++)
				s[i]=c.new Y();
		}
	}
}
