//S_u_v.java - S(u,v) selection block
//Copyright (c) 2007 Thomas Schneider
//See full Copyright license terms in file ../GPL.txt

package SFE.PF;

import SFE.BOAL.Gate;

/**
 * S(u,v) selection block.
 * <p>
 * Simple implementation using a S(u,1) selection block for each of the v outputs.
 *
 * @author Thomas Schneider
 * @see SFE.PF.S_u_1
 */
public class S_u_v extends S {
	S_u_1[] s;
	
	public int countGates() {
		return s[0].countGates()*v;
	}
	
	public int estimateGates(int u, int v) {
		return v*(u-1)*UC.Ygates;
	}

	public Gate[] extractCircuit(Gate[] inputs, Gate[] outputs) {
		if(outputs==null) outputs=new Gate[v];
		for(int i=0; i<v; i++) {
			outputs[i]=s[i].extractCircuit(inputs, null)[0];
		}
		return outputs;
	}

	public void program(int[] select) {
		for(int i=0; i<v; i++)
			s[i].program(select[i]);
	}
	
	public void generate(int u, int v, UC c) {
		this.u=u; this.v=v;
		s=new S_u_1[v];
		for(int i=0; i<v; i++) {
			s[i]=new S_u_1(u,c);
		}
	}
	
	public S_u_v(int u, int v, UC c) {
		super(u,v,c);
	}
	public S_u_v() {
		super();
	}
}
